function [L] = stratSampling( dirName, funcName, nDim , nStrata)
% dirName = 'DFP/Weieirstrass'
% funcName = Weierstrass
% nDim = 10
method = 'DFP';
% DFP_AllWeierstrass_10D.csv
fileName = strcat(dirName,'/',method,'_All',funcName,...
                        '_',num2str(nDim), 'D.csv');
fprintf('Processing %s\n',fileName);
A = load(fileName);
ind = stratIndices(A(:, nDim+1), nStrata, 50);
L = A(ind,:);
recordName = strcat(dirName,'/',method,'_',funcName,...
                        '_', num2str(nDim), 'D.csv');
dlmwrite(recordName,L,'delimiter',',','precision',16);
end

% Function to generate stratified indices
function [ output ] = stratIndices( X , nRequired, num_bin)
output = zeros(nRequired + num_bin,1);
N = length(X);
% Find the frequency of each strata
max_data = max(X);
min_data = min(X);
fprintf('min: %f. max: %f \n', min(X), max(X));
len = (max_data-min_data)/num_bin;
edges = min_data:len:max_data;
edges(1) = -inf;
edges(end) = inf;
[count bin] = histc(X, edges);
% Generate samples for each bin 
nData = 1;
rand('twister',951);
for i = 1:(num_bin)
    nSamples = ceil((count(i)/N) * nRequired);
    if (nSamples > 0)
        binInd = find(bin==i);
        strataInd = ceil(rand(nSamples,1)*length(binInd));
        output(nData:(nData + nSamples -1)) = binInd(strataInd);
        nData = nData + nSamples;
    end
end
output = output(output > 0);
end
