package mytest;

import java.util.Random;

import mytest.evaluation.FitnessFunction;
import mytest.evaluation.real.*;
import optimization.operator.individual.*;
import optimization.tools.Matrix;
import optimization.tools.Utils;

public class ConnectivityAnalysis {
	/**
	 * Program to generate sample point and run local
	 * search until it converges
	 */
	static public void main(String [] args)
	{
		FitnessFunction fc = new Weierstrass(0);
		System.out.println("java -jar file.jar [method] [filename]");
		// setting for run until converged
		IndivSearch operator;
		if (args[0].compareTo("DFP") == 0)  
			operator = new DFP(fc,-1);
		else
			operator = new DSCG(fc,-1);
		operator.StepSize = 0.5;
//		((DSCG) operator).debug = true;
		
//		double Radius = 10;
		double LRange = fc.getLowBound();
		double URange = fc.getUpBound();
		double [] v = new double[2];
		v[0] = 100*LRange; v[1] = 100*URange;
		fc.setDefaultRanges(v);
		
		int k = 0;
		/* **************************************************
		 * Generate initial point and output  
		 * **************************************************
		int nDim = 2;
		int N = 4000;
		double [][] localOptimums = new double[N][nDim+1];
		double [][] x = new double[N][nDim+1];
		double [] xtemp = new double[nDim];
		RandomGenerator rnd = new RandomGenerator();
		System.out.println("Generate initial and output");
		for (int r = 0; r < N; r++)
		{
			System.out.println("\tAt iteration " + r + "... ");
			// Local search started at random X
			for (int i = 0; i < nDim; i++) {
				xtemp[i] = LRange + rnd.getRandom(URange-LRange);
				x[r][i] = xtemp[i];
			}
			double initFit = fc.getFitnessFunc(xtemp);
			x[r][nDim] = initFit;
			// Start searching
			double currFit = operator.search(xtemp);
			// record the local point
			for (int i=0; i < nDim; i++)
				localOptimums[r][i] = xtemp[i];
			localOptimums[r][nDim] = currFit;
		}
		// Create matrix wrapper
		Matrix localMatrix = new Matrix(localOptimums);
		Matrix initMatrix = new Matrix(x);
		// print out local points
//		System.out.println("List of local optima found: ");
//		System.out.println(localMatrix.toString());
		
		// save to file
		Matrix.DMat2File(operator.name + "_" + fc.getName() + nDim + "DLocal_" + k + ".csv", localMatrix);
		Matrix.DMat2File(fc.getName() + nDim + "DInit_" + k + ".csv", initMatrix);
		*/
		
		/* **********************************************
		 * Read initialized point and generate output   
		 * **********************************************
		System.out.println("Read initial...");
		String filename = "Rastrigin2DInit_0.csv";
		Matrix initMatrix = Matrix.File2DMat(filename);
		System.out.println("Generate output");
		int nDim = initMatrix.col-1;
		int N = initMatrix.row;
		double [][] localOptimums = new double[N][nDim+1];
		double [][] x = new double[N][nDim+1];
		double [] xtemp = new double[nDim];
		for (int r = 0; r < N; r++)
		{
			System.out.println("\tAt iteration " + r + "... ");
			// Local search started at random X
			for (int i = 0; i < nDim; i++) {
				xtemp[i] = initMatrix.mat[r][i];
			}
			// Start searching
			double currFit = operator.search(xtemp);
			// record the local point
			for (int i=0; i < nDim; i++)
				localOptimums[r][i] = xtemp[i];
			localOptimums[r][nDim] = currFit;
		}
		// Create matrix wrapper
		Matrix localMatrix = new Matrix(localOptimums);
		// save to file
		Matrix.DMat2File(operator.name + "_" + fc.getName() + nDim + "DLocal_" + k + ".csv", localMatrix);
		*/
		
		/* **********************************************
		 * Read initialized point
		 * Mutate point T times at variance V 
		 * and generate fitness output   
		 * ********************************************** */
		System.out.println("Read initial from " + args[1] + "...");
		String filename = args[1];
		Matrix initMatrix = Matrix.File2DMat(filename);
		int N = initMatrix.row;
		int nDim = initMatrix.col - 1;
		
		int T = 10*nDim;	// T: number of iterations
		double V = 1;
		Random rd = new Random(System.currentTimeMillis());
		
		double [][] localOptimums = new double[N][nDim + 4];
		double [] xtemp = new double[nDim];
		for (int r = 0; r < N; r++)
		{
			double [] f_rate = new double[T];
			double [] i_rate = new double[T];
			double [] eval_rate = new double[T];
			double initFit = initMatrix.mat[r][nDim];
			System.out.println("\tAt iteration " + r + "... ");
			
			for (int t = 0; t < T; t++)
			{
//				System.out.println("At time " + t + "...");
				// Local search started at random X
				for (int i = 0; i < nDim; i++) {
					do {
						xtemp[i] = initMatrix.mat[r][i] + V*rd.nextGaussian();
        			} while (Utils.g(xtemp[i], URange) ||	Utils.l(xtemp[i], LRange));
				}
				// Before: evaluation
				long initEval = fc.getEvalCount();
				
				double ytemp = fc.getFitnessFunc(xtemp);
				// Start searching
				double currFit = operator.search(xtemp, ytemp);
				// Before: evaluation
				long afterEval = fc.getEvalCount();
				// Calculate progress rate
				f_rate[t] = (initFit - currFit);
				eval_rate[t] = (afterEval - initEval + 1);
				i_rate[t] = f_rate[t]/eval_rate[t];
			}
			// record the progress rate at the local point
			for (int i=0; i < nDim; i++)
				localOptimums[r][i] = initMatrix.mat[r][i];
			localOptimums[r][nDim] = initFit;
			localOptimums[r][nDim+1] = Utils.average(f_rate) ;
			localOptimums[r][nDim+2] = Utils.average(eval_rate);
			localOptimums[r][nDim+3] = Utils.average(i_rate);
		}
		System.out.println("Search method: " + args[0]
		                        + " on " + fc.getName() + nDim + "D");
		System.out.println("Generate output");
		// Create matrix wrapper
		Matrix localMatrix = new Matrix(localOptimums);
		// save to file
		Matrix.DMat2File("Var" + (new Double(V).toString())+ "_" +
				args[0] + "_" + fc.getName() + nDim + "DProgress_" + k + ".csv", localMatrix);
		
		
		/* **********************************************
		 * Read initialized point
		 * Mutate point T times at variance V 
		 * and generate distance output   
		 * ********************************************** 
		System.out.println("Read initial...");
		String filename = "Rastrigin_2D.csv";
		Matrix initMatrix = Matrix.File2DMat(filename);
		int T = 10;
		double V = 5;
		Random rd = new Random(System.currentTimeMillis());
		int N = initMatrix.row;
		int nDim = initMatrix.col - 1;
		double [][] localOptimums = new double[N][nDim+1];
		double [] xtemp = new double[nDim];
		double [] xinit = new double[nDim];
		for (int r = 0; r < N; r++)
		{
			double d_rate = 0;
			System.out.println("\tAt iteration " + r + "... ");
			for (int t = 0; t < T; t++)
			{
				// Local search started at random X
				for (int i = 0; i < nDim; i++) {
					xinit[i] = initMatrix.mat[r][i];
					xtemp[i] = initMatrix.mat[r][i] + V*rd.nextGaussian();
				}
				// Start searching
//				System.out.println("\t Start LS...");
				operator.search(xtemp);
				// Calculate progress rate
				d_rate = d_rate + MathExt.distance(xtemp, xinit);
			}
			// record the progress rate at the local point
			for (int i=0; i < nDim; i++)
				localOptimums[r][i] = initMatrix.mat[r][i];
			localOptimums[r][nDim] = d_rate/T ;
		}
		System.out.println("Generate output");
		// Create matrix wrapper
		Matrix localMatrix = new Matrix(localOptimums);
		// save to file
		Matrix.DMat2File("Var" + (new Double(V).toString())+ "_" +
				operator.name + "_" + fc.getName() + nDim + "DLocality_" + k + ".csv", localMatrix);
		*/
	}
}
