package mytest;

import mytest.evaluation.FitnessFunction;
import mytest.evaluation.real.*;
import optimization.operator.individual.*;
import optimization.tools.*;

public class LocalOptSampling {
	/**
	 * Program to generate R set of N samples toward
	 * converged local optimum
	 * @param args
	 */
	static public void main(String [] args)
	{
//		int nDim = 2;
		int nDim = Integer.parseInt(args[0]);
		if (nDim <= 0) 
		{
			System.err.println("Not enough arguments...");
			return;
		}
		// R = number of runs
		int R = 30;
		// N = number of samples
		int N = 4000;
		RandomGenerator rnd = new RandomGenerator();
		FitnessFunction fc = new Griewank(0);
		/* 
		 * setting for local learning running parameters until converged
		 */
		DFP operator = new DFP(fc,-1);
		operator.StepSize = 1.0;
		operator.CONV_ACC = 1E-5;
		operator.MAX_COUNT = 10;
		operator.debug = false;
		
		// Initialize function's range at double radius
		double LRange = fc.getLowBound();
		double URange = fc.getUpBound();
		double [] v = new double[2];
		v[0] = 2*LRange; v[1] = 2*URange;
		fc.setDefaultRanges(v);
		
		double [] x = new double[nDim];
		double [] xIdeal = new double[nDim];
		for (int i = 0; i < nDim; i++)
			xIdeal[i] = 0;
		System.out.println(xIdeal[0] +" " + xIdeal[1] + "-> " + fc.getFitnessFunc(xIdeal));
		
		for (int k = 0; k < R; k++) {
			System.out.println("At run " + k + "... ");
			double [][] localOptimums = new double[N][nDim+1];
			for (int r = 0; r < N; r++)
			{
				System.out.println("\tAt iteration " + r + "... ");
				// Local search started at random X
				for (int i = 0; i < nDim; i++)
					x[i] = LRange + rnd.getRandom(URange-LRange);
				double fx = fc.getFitnessFunc(x);
				// Searching for local optima
				double currFit = operator.search(x, fx);
				// Just copy for original sampling
//				double currFit = fx;
				// record the local point
				for (int i=0; i < nDim; i++)
					localOptimums[r][i] = x[i];
				localOptimums[r][nDim] = currFit;
			}
			// Create matrix wrapper
			Matrix localMatrix = new Matrix(localOptimums);
			
			// print out local points
//			System.out.println("List of local optimums found: ");
//			System.out.println(localMatrix.toString());
			
			// save to file
			Matrix.DMat2File(operator.name + "_" + fc.getName() + nDim + "DLocal_" + k + ".csv", localMatrix);
		}
	}

	
	
	
}
